@@ -55,7 +55,6 @@ struct _CtkPowermizer
     GtkWidget *adaptive_clock_status;
     GtkWidget *gpu_clock;
     GtkWidget *memory_transfer_rate;
-    GtkWidget *processor_clock;
     GtkWidget *power_source;
     GtkWidget *performance_level;
     GtkWidget *performance_table_hbox;
@@ -66,24 +65,18 @@ struct _CtkPowermizer
 
     gchar     *powermizer_menu_help;
 
-    GtkWidget *configuration_button;
-    gboolean  dp_enabled;
-    gboolean  dp_toggle_warning_dlg_shown;
     gboolean  hasDecoupledClock;
     gboolean  hasEditablePerfLevel;
     gboolean  editable_performance_levels_unified;
     gint      nvclock_attribute;
     gint      mem_transfer_rate_attribute;
-    gint      attribute;
     gint      powermizer_default_mode;
-    GtkWidget *status;
 
     GtkWidget *editable_perf_level_table;
     gint      num_perf_levels;
 
     GtkWidget *link_width;
     GtkWidget *link_speed;
-    gboolean  pcie_gen_queriable;
 };
 
 struct _CtkPowermizerClass
