@@ -222,6 +222,7 @@ static Bool LoadNvml(NvCtrlNvmlAttributes *nvml)
     GET_SYMBOL_REQUIRED(deviceGetIrqNum,                "nvmlDeviceGetIrqNum");
     GET_SYMBOL_REQUIRED(deviceGetPowerSource,           "nvmlDeviceGetPowerSource");
     GET_SYMBOL_REQUIRED(deviceGetNumFans,               "nvmlDeviceGetNumFans");
+    GET_SYMBOL_REQUIRED(deviceGetDefaultEccMode,        "nvmlDeviceGetDefaultEccMode");
 #undef GET_SYMBOL_REQUIRED
     
 /* Do not fail with older drivers */
@@ -229,6 +230,7 @@ static Bool LoadNvml(NvCtrlNvmlAttributes *nvml)
     nvml->lib._proc = dlsym(nvml->lib.handle, _name); 
     
     GET_SYMBOL_OPTIONAL(deviceGetGridLicensableFeatures, "nvmlDeviceGetGridLicensableFeatures_v4");
+    GET_SYMBOL_OPTIONAL(deviceGetGspFirmwareMode,        "nvmlDeviceGetGspFirmwareMode");
     GET_SYMBOL_OPTIONAL(deviceGetMemoryInfo_v2,          "nvmlDeviceGetMemoryInfo_v2");
 #undef GET_SYMBOL_OPTIONAL
 
@@ -936,6 +938,16 @@ static ReturnStatus NvCtrlNvmlGetGPUAttribute(const CtrlTarget *ctrl_target,
                 }
                 break;
 
+            case NV_CTRL_GPU_ECC_DEFAULT_CONFIGURATION:
+                {
+                    nvmlEnableState_t defaultMode;
+                    ret = nvml->lib.deviceGetDefaultEccMode(device, &defaultMode);
+                    if (ret == NVML_SUCCESS) {
+                        res = defaultMode;
+                    }
+                }
+                break;
+
             case NV_CTRL_GPU_ECC_SINGLE_BIT_ERRORS:
             case NV_CTRL_GPU_ECC_AGGREGATE_SINGLE_BIT_ERRORS:
             case NV_CTRL_GPU_ECC_DOUBLE_BIT_ERRORS:
@@ -989,7 +1001,6 @@ static ReturnStatus NvCtrlNvmlGetGPUAttribute(const CtrlTarget *ctrl_target,
                 ret = nvml->lib.deviceGetPowerSource(device, &res);
                 break;
 
-            case NV_CTRL_GPU_ECC_DEFAULT_CONFIGURATION:
             case NV_CTRL_VIDEO_RAM:
             case NV_CTRL_GPU_PCIE_MAX_LINK_SPEED:
             case NV_CTRL_GPU_PCIE_CURRENT_LINK_SPEED:
@@ -1127,6 +1138,53 @@ static ReturnStatus NvCtrlNvmlGetGridLicensableFeatures(const CtrlTarget *ctrl_t
     return NvCtrlNotSupported;
 }
 
+static ReturnStatus NvCtrlNvmlDeviceGetGspFeatures(const CtrlTarget *ctrl_target, int attr,
+                                                       unsigned int *isEnabled, unsigned int *defaultMode)
+{
+    const NvCtrlAttributePrivateHandle *h = getPrivateHandleConst(ctrl_target);
+    const NvCtrlNvmlAttributes *nvml;
+    nvmlDevice_t device;
+    nvmlReturn_t ret;
+
+    nvml = getNvmlHandleConst(h);
+    if (nvml == NULL) {
+        return NvCtrlBadHandle;
+    }
+
+    ret = nvml->lib.deviceGetHandleByIndex(nvml->deviceIdx, &device);
+    if (ret == NVML_SUCCESS) {
+    switch (attr) {
+        case NV_CTRL_ATTR_NVML_GSP_FIRMWARE_MODE:
+            if (nvml->lib.deviceGetGspFirmwareMode) {
+                unsigned int isEnabled_t = 0;
+                unsigned int defaultMode_t = 0;
+                ret = nvml->lib.deviceGetGspFirmwareMode(device,
+                                                         &isEnabled_t, &defaultMode_t);
+                if (ret == NVML_SUCCESS) {
+                    *isEnabled = isEnabled_t;
+                    *defaultMode = defaultMode_t;
+                    return NvCtrlSuccess;
+                }
+            } else {
+                /* return NvCtrlNotSupported against older driver */
+                ret = NVML_ERROR_FUNCTION_NOT_FOUND;
+            }
+
+            break;
+
+        default:
+            /* Did we forget to handle a GPU integer attribute? */
+            nv_warning_msg("Unhandled integer attribute %s (%d) of GPU "
+                           "(%d)", INT_ATTRIBUTE_NAME(attr), attr,
+                           NvCtrlGetTargetId(ctrl_target));
+            return NvCtrlNotSupported;
+        }
+    }
+
+    /* An NVML error occurred */
+    printNvmlError(ret);
+    return NvCtrlNotSupported;
+}
 
 static void getDeviceAndTargetIndex(const NvCtrlAttributePrivateHandle *h,
                                     unsigned int targetCount,
@@ -1320,6 +1378,29 @@ ReturnStatus NvCtrlNvmlGetGridLicenseAttributes(const CtrlTarget *ctrl_target,
 }
 
 
+ReturnStatus NvCtrlNvmlDeviceGetGspAttributes(const CtrlTarget *ctrl_target, int attr,
+                                              unsigned int *isEnabled, unsigned int *defaultMode)
+{
+    if (NvmlMissing(ctrl_target)) {
+        return NvCtrlMissingExtension;
+    }
+
+    /*
+     * This shouldn't be reached for target types that are not handled through
+     * NVML (Keep TARGET_TYPE_IS_NVML_COMPATIBLE in NvCtrlAttributesPrivate.h up
+     * to date).
+     */
+    assert(TARGET_TYPE_IS_NVML_COMPATIBLE(NvCtrlGetTargetType(ctrl_target)));
+
+    if (NvCtrlGetTargetType(ctrl_target) == GPU_TARGET) {
+        return NvCtrlNvmlDeviceGetGspFeatures(ctrl_target, attr, isEnabled, defaultMode);
+    }
+    else {
+            return NvCtrlBadHandle;
+    }
+}
+
+
 /*
  * Set NVML Attribute Values
  */
@@ -1838,6 +1919,7 @@ NvCtrlNvmlGetGPUValidAttributeValues(const CtrlTarget *ctrl_target, int attr,
             case NV_CTRL_GPU_ECC_STATUS:
             case NV_CTRL_GPU_ECC_CONFIGURATION:
             case NV_CTRL_GPU_ECC_CONFIGURATION_SUPPORTED:
+            case NV_CTRL_GPU_ECC_DEFAULT_CONFIGURATION:
                 val->valid_type = CTRL_ATTRIBUTE_VALID_TYPE_BOOL;
                 break;
 
@@ -1852,7 +1934,6 @@ NvCtrlNvmlGetGPUValidAttributeValues(const CtrlTarget *ctrl_target, int attr,
                 val->valid_type = CTRL_ATTRIBUTE_VALID_TYPE_BITMASK;
                 break;
 
-            case NV_CTRL_GPU_ECC_DEFAULT_CONFIGURATION:
             case NV_CTRL_VIDEO_RAM:
             case NV_CTRL_GPU_PCIE_MAX_LINK_SPEED:
             case NV_CTRL_GPU_PCIE_CURRENT_LINK_SPEED:
